@@ -15,15 +15,16 @@
 
 import abc
 
+from neutron_lib.api import extensions as api_extensions
+from neutron_lib.db import constants as db_const
 from neutron_lib import exceptions as nexception
+from neutron_lib.plugins import directory
 
 from midonet.neutron._i18n import _
 from midonet.neutron.common import constants
 from neutron.api import extensions
-from neutron.api.v2 import attributes as attr
 from neutron.api.v2 import base
 from neutron.api.v2 import resource_helper
-from neutron import manager
 from oslo_log import log as logging
 import six
 
@@ -127,7 +128,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                'is_visible': True,
                'primary_key': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                  'default': "",
                  'is_visible': True},
         'type': {'allow_post': True, 'allow_put': False,
@@ -136,7 +137,8 @@ RESOURCE_ATTRIBUTE_MAP = {
                  'is_visible': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'required_by_policy': True,
-                      'validate': {'type:string': attr.TENANT_ID_MAX_LEN},
+                      'validate': {'type:string':
+                          db_const.PROJECT_ID_FIELD_SIZE},
                       'is_visible': True},
         'management_ip': {'allow_post': True, 'allow_put': False,
                           'default': None,
@@ -148,7 +150,8 @@ RESOURCE_ATTRIBUTE_MAP = {
         'management_protocol': {'allow_post': True, 'allow_put': False,
                                 'is_visible': True, 'default': None},
         'resource_id': {'allow_post': True, 'allow_put': False,
-                        'validate': {'type:string': attr.DEVICE_ID_MAX_LEN},
+                        'validate': {'type:string':
+                            db_const.DEVICE_ID_FIELD_SIZE},
                         'is_visible': True, 'required_by_policy': True,
                         'default': ""},
         'tunnel_ips': {'allow_post': True, 'allow_put': True,
@@ -189,13 +192,14 @@ SUB_RESOURCE_ATTRIBUTE_MAP = {
             #    if neutron core is not modified.
             'tenant_id': {'allow_post': True, 'allow_put': False,
                           'required_by_policy': True,
-                          'validate': {'type:string': attr.TENANT_ID_MAX_LEN},
+                          'validate': {'type:string':
+                              db_const.PROJECT_ID_FIELD_SIZE},
                           'is_visible': False}}
     }
 }
 
 
-class Gateway_device(extensions.ExtensionDescriptor):
+class Gateway_device(api_extensions.ExtensionDescriptor):
     """Gateway device extension."""
 
     @classmethod
@@ -225,13 +229,11 @@ class Gateway_device(extensions.ExtensionDescriptor):
         plural_mappings = resource_helper.build_plural_mappings(
             {}, RESOURCE_ATTRIBUTE_MAP)
 
-        attr.PLURALS.update(plural_mappings)
         resources = resource_helper.build_resource_info(
             plural_mappings,
             RESOURCE_ATTRIBUTE_MAP,
             constants.GATEWAY_DEVICE)
-        plugin = manager.NeutronManager.get_service_plugins()[
-            constants.GATEWAY_DEVICE]
+        plugin = directory.get_plugin(constants.GATEWAY_DEVICE)
 
         for collection_name in SUB_RESOURCE_ATTRIBUTE_MAP:
             # Special handling needed for sub-resources with 'y' ending
